#!/usr/bin/env python3
"""
Network Traffic Analyzer
Advanced network traffic analysis tool with packet inspection and statistics.
Parameters:
    - interface: Network interface to monitor
    - mode: Analysis mode (basic, advanced, forensic)
    - output: Output file for analysis results
Requires: scapy package and admin/root privileges
"""
from scapy.all import *
import argparse
import logging
import json
from datetime import datetime
import collections
from pathlib import Path
import threading
import queue

class NetworkTrafficAnalyzer:
    def __init__(self, interface, mode='basic', output=None):
        self.interface = interface
        self.mode = mode
        self.output_file = output
        self.packet_queue = queue.Queue()
        self.stop_event = threading.Event()
        self.stats = {
            'total_packets': 0,
            'protocols': collections.Counter(),
            'ports': collections.Counter(),
            'ip_addresses': collections.Counter()
        }
        self.setup_logging()

    def setup_logging(self):
        logging.basicConfig(
            level=logging.INFO,
            format='%(asctime)s - %(levelname)s - %(message)s'
        )
        self.logger = logging.getLogger(__name__)

    def packet_callback(self, packet):
        self.packet_queue.put(packet)
        self.stats['total_packets'] += 1

        if IP in packet:
            self.stats['ip_addresses'][packet[IP].src] += 1
            self.stats['ip_addresses'][packet[IP].dst] += 1

            if TCP in packet:
                self.stats['protocols']['TCP'] += 1
                self.stats['ports'][packet[TCP].sport] += 1
                self.stats['ports'][packet[TCP].dport] += 1
            elif UDP in packet:
                self.stats['protocols']['UDP'] += 1
                self.stats['ports'][packet[UDP].sport] += 1
                self.stats['ports'][packet[UDP].dport] += 1
            elif ICMP in packet:
                self.stats['protocols']['ICMP'] += 1

    def analyze_traffic(self):
        try:
            self.logger.info(f"Starting traffic analysis on interface {self.interface}")
            sniff(iface=self.interface, prn=self.packet_callback, 
                  store=0, stop_filter=lambda _: self.stop_event.is_set())
        except Exception as e:
            self.logger.error(f"Error during traffic analysis: {str(e)}")

    def save_results(self):
        if self.output_file:
            try:
                results = {
                    'timestamp': datetime.now().isoformat(),
                    'interface': self.interface,
                    'mode': self.mode,
                    'statistics': {
                        'total_packets': self.stats['total_packets'],
                        'top_protocols': dict(self.stats['protocols'].most_common(10)),
                        'top_ports': dict(self.stats['ports'].most_common(10)),
                        'top_ip_addresses': dict(self.stats['ip_addresses'].most_common(10))
                    }
                }
                
                with open(self.output_file, 'w') as f:
                    json.dump(results, f, indent=4)
                self.logger.info(f"Analysis results saved to {self.output_file}")
            except Exception as e:
                self.logger.error(f"Error saving results: {str(e)}")

    def start_analysis(self):
        analysis_thread = threading.Thread(target=self.analyze_traffic)
        analysis_thread.start()
        
        try:
            while True:
                time.sleep(1)
        except KeyboardInterrupt:
            self.logger.info("Stopping traffic analysis...")
            self.stop_event.set()
            analysis_thread.join()
            self.save_results()

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Network Traffic Analyzer")
    parser.add_argument("interface", help="Network interface to monitor")
    parser.add_argument("--mode", choices=['basic', 'advanced', 'forensic'],
                      default='basic', help="Analysis mode")
    parser.add_argument("--output", help="Output file for analysis results")
    args = parser.parse_args()

    analyzer = NetworkTrafficAnalyzer(args.interface, args.mode, args.output)
    analyzer.start_analysis()
